
import { NextResponse } from 'next/server';
import type { NextRequest } from 'next/server';
import { addFile } from '@/lib/file-service';
import { revalidatePath } from 'next/cache';
import { z } from 'zod';
import fs from 'node:fs';
import path from 'node:path';
import { UPLOAD_LIMIT_MB } from '@/lib/config';

const MAX_FILE_SIZE_BYTES = UPLOAD_LIMIT_MB * 1024 * 1024;

// This schema can be shared or adapted from src/lib/actions/file.actions.ts
const fileSchema = z.object({
    file: z
        .custom<File>((val) => val instanceof File, "Input is not a file")
        .refine((file) => file.size > 0, "File cannot be empty.")
        .refine((file) => file.size <= MAX_FILE_SIZE_BYTES, `File size should be less than ${UPLOAD_LIMIT_MB}MB.`)
    // You might want to add more specific MIME type validation if needed
    // .refine((file) => ACCEPTED_FILE_TYPES_REGEX.test(file.type), "Unsupported file type.")
});

export async function POST(request: NextRequest) {
    try {
        const formData = await request.formData();
        const uploadedFile = formData.get('file') as File | null;

        if (!uploadedFile || uploadedFile.name === 'undefined' || uploadedFile.size === 0) {
            return NextResponse.json({ message: 'No file selected or file is empty.', error: true }, { status: 400 });
        }

        const validatedFields = fileSchema.safeParse({ file: uploadedFile });

        if (!validatedFields.success) {
            const errors = validatedFields.error.flatten().fieldErrors;
            return NextResponse.json({
                message: errors.file?.[0] || "Invalid file.",
                error: true,
            }, { status: 400 });
        }

        const newFileEntry = await addFile({
            name: uploadedFile.name,
            type: uploadedFile.type || 'application/octet-stream',
            size: uploadedFile.size,
        });

        const cdnFolderPath = path.join(process.cwd(), 'CDN');
        if (!fs.existsSync(cdnFolderPath)) {
            fs.mkdirSync(cdnFolderPath, { recursive: true });
        }

        const filePathInCdn = path.join(cdnFolderPath, `${newFileEntry.id}.${newFileEntry.extension}`);
        const fileBuffer = Buffer.from(await uploadedFile.arrayBuffer());
        fs.writeFileSync(filePathInCdn, fileBuffer);

        // Revalidate paths that show file lists or file details
        revalidatePath('/browse', 'page');
        revalidatePath(`/files/${newFileEntry.id}`, 'page');
        // Revalidating the CDN path might be less critical if content-disposition/cache headers are set well client-side
        // but can be included for thoroughness.
        revalidatePath(`/cdn/${newFileEntry.id}`, 'page');

        return NextResponse.json({
            message: `File "${newFileEntry.name}" uploaded and saved successfully!`,
            error: false,
            file: newFileEntry
        }, { status: 200 });

    } catch (e) {
        console.error("Upload API error:", e);
        let errorMessage = 'Failed to upload file via API. Please try again.';
        if (e instanceof Error) {
            errorMessage = `Failed to upload file: ${e.message}`;
        }
        return NextResponse.json({ message: errorMessage, error: true }, { status: 500 });
    }
}
