
import { NextResponse } from 'next/server';
import type { NextRequest } from 'next/server';
import { getFileById } from '@/lib/file-service';
import fs from 'node:fs';
import path from 'node:path';

export async function GET(
  request: NextRequest,
  { params }: { params: { fileId: string } }
) {
  const fileId = params.fileId;

  if (!fileId) {
    return NextResponse.json({ error: 'File ID is required' }, { status: 400 });
  }

  const fileMetadata = await getFileById(fileId);

  if (!fileMetadata) {
    return NextResponse.json({ error: 'File metadata not found or ID is incorrect' }, { status: 404 });
  }

  // Construct the full path to the file in the CDN directory
  // The filename on disk is [id].[extension]
  const cdnDirectory = path.join(process.cwd(), 'CDN');
  const filePath = path.join(cdnDirectory, `${fileMetadata.id}.${fileMetadata.extension}`);

  if (!fs.existsSync(filePath)) {
    console.error(`File not found at path: ${filePath} for ID ${fileId}`);
    return NextResponse.json({ error: 'File not found in CDN storage' }, { status: 404 });
  }

  try {
    const fileBuffer = fs.readFileSync(filePath);
    
    const headers = new Headers();
    headers.set('Content-Type', fileMetadata.type || 'application/octet-stream');
    // Use 'inline' for display in browser, 'attachment' to force download
    headers.set('Content-Disposition', `inline; filename="${encodeURIComponent(fileMetadata.name)}"`); 
    headers.set('Content-Length', fileMetadata.size.toString());

    return new NextResponse(fileBuffer, {
      status: 200,
      headers: headers,
    });

  } catch (error) {
    console.error(`Error reading file ${fileMetadata.id}.${fileMetadata.extension}:`, error);
    return NextResponse.json({ error: 'Error serving file' }, { status: 500 });
  }
}
