
import { getFileById } from '@/lib/file-service';
import { notFound } from 'next/navigation';
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from '@/components/ui/card';
import { Button } from '@/components/ui/button';
import Link from 'next/link';
import { ArrowLeft, DownloadCloud, ExternalLink } from 'lucide-react'; // Added ExternalLink
import { format } from 'date-fns';
import { FileTypeIcon } from '@/components/icons/FileTypeIcon';
import { Badge } from '@/components/ui/badge'; // Added Badge

interface FileDetailsPageProps {
  params: {
    fileId: string;
  };
}

function formatFileSize(bytes: number): string {
  if (bytes === 0) return '0 Bytes';
  const k = 1024;
  const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
  const i = Math.floor(Math.log(bytes) / Math.log(k));
  return parseFloat((bytes / Math.pow(k, i)).toFixed(2)) + ' ' + sizes[i];
}

export default async function FileDetailsPage({ params }: FileDetailsPageProps) {
  const file = await getFileById(params.fileId);

  if (!file) {
    notFound();
  }

  const cdnUrl = `/cdn/${file.id}`;
  // In a real app with a proper domain, you might construct the full URL:
  // const fullCdnUrl = typeof window !== 'undefined' ? `${window.location.origin}${cdnUrl}` : cdnUrl;
  // For server components, we can't rely on window.location.origin easily without passing it or client component tricks.
  // So we'll display the relative path which is still useful.

  return (
    <div className="max-w-3xl mx-auto">
      <Button variant="outline" asChild className="mb-6 group">
        <Link href="/browse">
          <ArrowLeft className="mr-2 h-4 w-4 group-hover:-translate-x-1 transition-transform" />
          Back to Files
        </Link>
      </Button>
      <Card className="shadow-xl overflow-hidden">
        <CardHeader className="bg-muted/30 p-6">
          <div className="flex flex-col sm:flex-row items-start sm:items-center space-y-3 sm:space-y-0 sm:space-x-4">
            <FileTypeIcon fileType={file.type} extension={file.extension} className="w-16 h-16 sm:w-20 sm:h-20 text-primary shrink-0" />
            <div className="flex-grow min-w-0">
              <CardTitle className="text-2xl sm:text-3xl font-bold tracking-tight font-headline break-words">{file.name}</CardTitle>
              <CardDescription className="text-sm sm:text-base mt-1">
                Unique ID: <Badge variant="secondary" className="font-mono">{file.id}</Badge>
              </CardDescription>
            </div>
          </div>
        </CardHeader>
        <CardContent className="p-6 space-y-6">
          <div className="grid grid-cols-1 md:grid-cols-2 gap-x-8 gap-y-4 text-base">
            <div>
              <strong className="font-semibold text-foreground">File Size:</strong>
              <p className="text-muted-foreground">{formatFileSize(file.size)}</p>
            </div>
            <div>
              <strong className="font-semibold text-foreground">MIME Type:</strong>
              <p className="text-muted-foreground">{file.type || 'N/A'}</p>
            </div>
            <div>
              <strong className="font-semibold text-foreground">Uploaded:</strong>
              <p className="text-muted-foreground">{format(new Date(file.uploadDate), "PPPp")}</p>
            </div>
            <div>
              <strong className="font-semibold text-foreground">Extension:</strong>
              <p className="text-muted-foreground">.{file.extension}</p>
            </div>
          </div>
          
          <div className="border-t border-border pt-6">
            <strong className="font-semibold text-foreground text-lg">Direct CDN Link:</strong>
            <div className="mt-2 flex items-center gap-2">
              <div className="flex-grow p-3 bg-muted rounded-md text-sm text-muted-foreground overflow-x-auto">
                <code className="font-mono">{cdnUrl}</code>
              </div>
              <Button variant="outline" size="icon" asChild>
                <a href={cdnUrl} target="_blank" rel="noopener noreferrer" aria-label="Open CDN link in new tab">
                  <ExternalLink className="h-4 w-4" />
                </a>
              </Button>
            </div>
            <p className="text-xs text-muted-foreground mt-1">This is the direct link to access the file content.</p>
            <p className="text-xs text-muted-foreground mt-0.5"> (For full URL, prefix with your application's domain, e.g., https://yourdomain.com{cdnUrl})</p>
          </div>
          
          <Button size="lg" className="w-full mt-4" asChild>
            <a href={cdnUrl} download={file.name}>
              <DownloadCloud className="mr-2 h-5 w-5" />
              Download File
            </a>
          </Button>
        </CardContent>
      </Card>
    </div>
  );
}

