
"use client";

import { useRef, useState, useEffect, FormEvent } from 'react';
import { Button } from '@/components/ui/button';
import { Input } from '@/components/ui/input';
import { Label } from '@/components/ui/label';
import { Progress } from '@/components/ui/progress';
import { useToast } from '@/hooks/use-toast';
import { Alert, AlertDescription, AlertTitle } from "@/components/ui/alert";
import { UploadCloud, CheckCircle, XCircle, Loader2, File as FileIconLucide } from "lucide-react";
import { UPLOAD_LIMIT_MB } from '@/lib/config';
import type { FileItem } from '@/types';

interface UploadMessage {
  text: string | null;
  type: 'success' | 'error' | null;
  uploadedFile?: FileItem | null;
}

function SubmitButton({ isUploading }: { isUploading: boolean }) {
  return (
      <Button type="submit" className="w-full mt-4" disabled={isUploading}>
        {isUploading ? <Loader2 className="mr-2 h-4 w-4 animate-spin" /> : <UploadCloud className="mr-2 h-4 w-4" />}
        Upload File
      </Button>
  );
}

export default function FileUploadForm() {
  const { toast } = useToast();
  const [selectedFile, setSelectedFile] = useState<File | null>(null);
  const [uploadProgress, setUploadProgress] = useState(0);
  const [isUploading, setIsUploading] = useState(false);
  const [message, setMessage] = useState<UploadMessage>({ text: null, type: null, uploadedFile: null });
  const formRef = useRef<HTMLFormElement>(null);
  const fileInputRef = useRef<HTMLInputElement>(null);

  const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (file) {
      if (file.size > UPLOAD_LIMIT_MB * 1024 * 1024) {
        setMessage({ text: `File size exceeds ${UPLOAD_LIMIT_MB}MB limit.`, type: 'error' });
        setSelectedFile(null);
        if(fileInputRef.current) fileInputRef.current.value = ""; // Clear the input
        return;
      }
      setSelectedFile(file);
      setUploadProgress(0);
      setMessage({ text: null, type: null });
    } else {
      setSelectedFile(null);
    }
  };

  const handleSubmit = async (event: FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    if (!selectedFile) {
      setMessage({ text: 'Please select a file to upload.', type: 'error' });
      return;
    }

    setIsUploading(true);
    setUploadProgress(0);
    setMessage({ text: null, type: null });

    const formData = new FormData();
    formData.append('file', selectedFile);

    const xhr = new XMLHttpRequest();

    xhr.upload.onprogress = (event) => {
      if (event.lengthComputable) {
        const percentComplete = Math.round((event.loaded / event.total) * 100);
        setUploadProgress(percentComplete);
      }
    };

    xhr.onload = () => {
      setIsUploading(false);
      try {
        const response = JSON.parse(xhr.responseText);
        if (xhr.status >= 200 && xhr.status < 300 && !response.error) {
          setMessage({ text: response.message || "File uploaded successfully!", type: 'success', uploadedFile: response.file });
          formRef.current?.reset();
          setSelectedFile(null);
          // Keep progress at 100 for a bit for visual feedback
          setUploadProgress(100);
          setTimeout(() => setUploadProgress(0), 2000);
        } else {
          setMessage({ text: response.message || `Upload failed with status: ${xhr.status}`, type: 'error' });
          setUploadProgress(0);
        }
      } catch (e) {
        setMessage({ text: 'An unexpected error occurred during upload processing.', type: 'error' });
        setUploadProgress(0);
      }
    };

    xhr.onerror = () => {
      setIsUploading(false);
      setMessage({ text: 'Upload failed. Please check your network connection and try again.', type: 'error' });
      setUploadProgress(0);
    };

    xhr.open('POST', '/api/upload', true);
    xhr.send(formData);
  };

  useEffect(() => {
    if (message.text) {
      if (message.type === 'error') {
        toast({
          variant: "destructive",
          title: "Upload Failed",
          description: message.text,
          icon: <XCircle className="h-5 w-5" />,
        });
      } else if (message.type === 'success') {
        toast({
          title: "Upload Successful!",
          description: message.text,
          icon: <CheckCircle className="h-5 w-5 text-green-500" />,
        });
      }
    }
  }, [message, toast]);

  return (
      <form onSubmit={handleSubmit} ref={formRef} className="space-y-6">
        <div>
          <Label htmlFor="file-upload" className="sr-only">Choose file</Label>
          <div className="mt-2 flex justify-center rounded-lg border border-dashed border-border px-6 py-10 hover:border-primary transition-colors duration-200">
            <div className="text-center">
              {selectedFile ? (
                  <FileIconLucide className="mx-auto h-12 w-12 text-muted-foreground" aria-hidden="true" />
              ) : (
                  <UploadCloud className="mx-auto h-12 w-12 text-muted-foreground" aria-hidden="true" />
              )}
              <div className="mt-4 flex text-sm leading-6 text-muted-foreground">
                <label
                    htmlFor="file-upload"
                    className="relative cursor-pointer rounded-md font-semibold text-primary focus-within:outline-none focus-within:ring-2 focus-within:ring-primary focus-within:ring-offset-2 hover:text-accent"
                >
                  <span>{selectedFile ? "Change file" : "Upload a file"}</span>
                  <Input ref={fileInputRef} id="file-upload" name="file" type="file" className="sr-only" onChange={handleFileChange} required />
                </label>
                <p className="pl-1">{selectedFile ? "" : "or drag and drop"}</p>
              </div>
              {selectedFile ? (
                  <p className="text-xs leading-5 text-muted-foreground mt-1">{selectedFile.name} ({(selectedFile.size / 1024 / 1024).toFixed(2)} MB)</p>
              ) : (
                  <p className="text-xs leading-5 text-muted-foreground">PNG, JPG, PDF, MP4 up to {UPLOAD_LIMIT_MB}MB</p>
              )}
            </div>
          </div>
        </div>

        {selectedFile && (isUploading || uploadProgress > 0) && (
            <div className="space-y-2">
              <Progress value={uploadProgress} className="w-full h-3" />
              <p className="text-sm text-muted-foreground text-center">
                {isUploading && uploadProgress < 100 ? `${uploadProgress}% uploading...` : message.type === 'error' ? 'Error' : uploadProgress === 100 && !isUploading && message.type === 'success' ? 'Complete' : `${uploadProgress}%`}
              </p>
            </div>
        )}

        {message.type === 'error' && message.text && !isUploading && (
            <Alert variant="destructive">
              <XCircle className="h-4 w-4" />
              <AlertTitle>Error</AlertTitle>
              <AlertDescription>{message.text}</AlertDescription>
            </Alert>
        )}

        <SubmitButton isUploading={isUploading} />
      </form>
  );
}
