
import { NextResponse } from 'next/server';
import type { NextRequest } from 'next/server';
import { AUTH_COOKIE_NAME } from '@/lib/constants';

const PROTECTED_ROUTES = ['/browse', '/upload'];
// Added /cdn to public routes
const PUBLIC_ROUTES = ['/login', '/files', '/cdn']; 

export function middleware(request: NextRequest) {
  const { pathname } = request.nextUrl;
  const sessionCookie = request.cookies.get(AUTH_COOKIE_NAME);
  const isAuthenticated = sessionCookie?.value === 'true';

  const isProtectedRoute = PROTECTED_ROUTES.some(route => pathname.startsWith(route));
  
  // Check if the path starts with any of the public routes patterns
  const isPublicRoutePattern = PUBLIC_ROUTES.some(publicRoute => {
    if (publicRoute.endsWith('/[...slug]') || publicRoute.endsWith('/[param]')) {
      // For dynamic routes, check if the pathname starts with the base of the public route
      // e.g., /files/* or /cdn/*
      const publicRouteBase = publicRoute.substring(0, publicRoute.indexOf('['));
      return pathname.startsWith(publicRouteBase);
    }
    return pathname.startsWith(publicRoute);
  });


  if (isProtectedRoute && !isAuthenticated) {
    return NextResponse.redirect(new URL('/login', request.url));
  }

  if (pathname === '/login' && isAuthenticated) {
    return NextResponse.redirect(new URL('/browse', request.url));
  }
  
  // Allow access to root path and routes matching public patterns regardless of authentication
  if (pathname === '/' || isPublicRoutePattern) {
    return NextResponse.next();
  }

  // If it's not a public route and not a protected route (already handled if not authenticated),
  // or if it's a protected route and authenticated, allow access.
  // This simplifies logic as specific redirects are handled above.
  return NextResponse.next();
}

export const config = {
  matcher: [
    /*
     * Match all request paths except for the ones starting with:
     * - api (API routes)
     * - _next/static (static files)
     * - _next/image (image optimization files)
     * - favicon.ico (favicon file)
     */
    '/((?!api|_next/static|_next/image|favicon.ico).*)',
  ],
};
